package examples.alarmclock;

import java.util.Arrays;

import org.apache.log4j.Logger;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;
import org.ungoverned.gravity.servicebinder.Lifecycle;
import org.ungoverned.gravity.servicebinder.ServiceBinderContext;

import com.francetelecom.amigo.core.AmigoException;
import com.francetelecom.amigo.core.AmigoLdapLookup;
import com.francetelecom.amigo.core.AmigoService;

import fr.inria.icocoa.common.PropertyManager;
import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.semrep.CapabilityInvoker;
import fr.inria.icocoa.semrep.SemanticRepository;

public class SemanticRepositoryClientComponent implements Lifecycle {

	private Logger log;
	private AmigoLdapLookup lookup;
	private BundleContext context;

	public SemanticRepositoryClientComponent(ServiceBinderContext sbc) {
		log = Logger.getLogger(this.getClass().getName());
		context = sbc.getBundleContext();
	}

	public void bindLookupService(AmigoLdapLookup lookup) {
		this.lookup = lookup;
	}

	public void unbindLookupService(AmigoLdapLookup lookup) {
		this.lookup = lookup;
	}

	public void activate() {
		log.info("Activating Semantic Repository Client Service...");
		if (lookup != null) {
			AmigoService[] services;
			try {
				services = lookup.lookup("urn:amigo",
						"SemanticServiceRepository", 5);

				if (services.length == 0) {
					log
							.warn("Unable to find services matching 'SemanticServiceRepository'.");
				} else if (services.length > 1) {
					log.error("Found multiple semantic repositories: "
							+ Arrays.toString(services));
				} else {
					SemanticRepository semrep = (SemanticRepository) services[0]
							.getSpecificStub(SemanticRepository.class);

					try {
						String taskFileUri = Tools
								.getURIStringFromFilePath(PropertyManager.LOCAL_DIRECTORY
										+ "/owl/AlarmClock.owl");

						log.info("Discovering services...");
						String[] matchingServices = semrep
								.getServicesForTask(taskFileUri);

						if (matchingServices.length > 0) {
							String serviceWsdlUri = semrep
									.getWsdlForService(matchingServices[0]);

							log.info("Activating alarm clock (invoker)...");
							CapabilityInvoker invoker = new CapabilityInvoker(
									serviceWsdlUri);
							invoker.invoke("AlarmClockActivate");

							log.info("Activating alarm clock (stub)...");
							AlarmClock alarmClock = new AlarmClockStub(
									serviceWsdlUri);
							alarmClock.activate();

						} else {
							log.info("No services were found matching "
									+ taskFileUri + ".");
						}
					} catch (Exception e) {
						// Handle exception
					}
				}
			} catch (AmigoException e) {
				log
						.warn("Lookup failed for services of type 'SemanticServiceRepository'.");
			}
		} else {
			log
					.warn("No lookup service found for Semantic Repository Client bundle.");
		}

		new Thread() {
			public void run() {
				try {
					context.getBundle().stop();
				} catch (BundleException e) {
					log.warn("Unable to stop bundle.", e);
				}
			}
		}.start();

	}

	public void deactivate() {
		log.info("Stopping the Semantic Repository Client bundle.");
	}

}
